/*
 * File:   dds2.c
 * Author: Pavel Dvok
 *
 * Created on July 2, 2016, 3:24 PM
 */
#pragma config FOSC = IRC    // Oscillator Selection bits->HS oscillator
#pragma config PLLEN = OFF    // 4 X PLL Enable bit->PLL is under software control
#pragma config PCLKEN = ON    // Primary Clock Enable bit->Primary clock enabled
#pragma config FCMEN = OFF    // Fail-Safe Clock Monitor Enable->Fail-Safe Clock Monitor disabled
#pragma config IESO = OFF    // Internal/External Oscillator Switchover bit->Oscillator Switchover mode disabled

// CONFIG2L
#pragma config PWRTEN = ON    // Power-up Timer Enable bit->PWRT enabled
#pragma config BOREN = SBORDIS    // Brown-out Reset Enable bits->Brown-out Reset enabled in hardware only (SBOREN is disabled)
#pragma config BORV = 19    // Brown Out Reset Voltage bits->VBOR set to 1.9 V nominal

// CONFIG2H
#pragma config WDTEN = OFF    // Watchdog Timer Enable bit->WDT is controlled by SWDTEN bit of the WDTCON register
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits->1:32768

// CONFIG3H
#pragma config HFOFST = ON    // HFINTOSC Fast Start-up bit->The system clock is held off until the HFINTOSC is stable.
#pragma config MCLRE = OFF    // MCLR Pin Enable bit->RA3 input pin enabled; MCLR disabled

// CONFIG4L
#pragma config STVREN = ON    // Stack Full/Underflow Reset Enable bit->Stack full/underflow will cause Reset
#pragma config LVP = OFF    // Single-Supply ICSP Enable bit->Single-Supply ICSP disabled
#pragma config BBSIZ = OFF    // Boot Block Size Select bit->1kW boot block size
#pragma config XINST = OFF    // Extended Instruction Set Enable bit->Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
#pragma config DEBUG = OFF    // Background Debugger Enable bit->Background debugger disabled, RA0 and RA1 configured as general purpose I/O pins

// CONFIG5L
#pragma config CP0 = OFF    // Code Protection bit->Block 0 not code-protected
#pragma config CP1 = OFF    // Code Protection bit->Block 1 not code-protected

// CONFIG5H
#pragma config CPB = OFF    // Boot Block Code Protection bit->Boot block not code-protected
#pragma config CPD = OFF    // Data EEPROM Code Protection bit->Data EEPROM not code-protected

// CONFIG6L
#pragma config WRT0 = OFF    // Write Protection bit->Block 0 not write-protected
#pragma config WRT1 = OFF    // Write Protection bit->Block 1 not write-protected

// CONFIG6H
#pragma config WRTC = OFF    // Configuration Register Write Protection bit->Configuration registers not write-protected
#pragma config WRTB = OFF    // Boot Block Write Protection bit->Boot block not write-protected
#pragma config WRTD = OFF    // Data EEPROM Write Protection bit->Data EEPROM not write-protected

// CONFIG7L
#pragma config EBTR0 = OFF    // Table Read Protection bit->Block 0 not protected from table reads executed in other blocks
#pragma config EBTR1 = OFF    // Table Read Protection bit->Block 1 not protected from table reads executed in other blocks

// CONFIG7H
#pragma config EBTRB = OFF    // Boot Block Table Read Protection bit->Boot block not protected from table reads executed in other blocks


#include <xc.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#include "displ2.c"

#define RS PORTAbits.RA1  //display text - command  
#define EN PORTCbits.RC4  //137  enable 
#define LDD PORTAbits.RA5 //load 137 - GL 

#define CMD 0  //command
#define DAT 1    //data
#define BFS 64   // FIFO
#define EPB  0    // eeprom
#define SCLK PORTBbits.RB6 
#define SDO  PORTCbits.RC7 

#define LCD  0    //displej
#define CST  1    //tlatka
#define SPIE  2   //extern SPI
#define FQ    3    //FQ DDS
#define TXON  4     //TX enable
#define RDDS  5     // reset DDS
#define CS752 6     // PIC752
#define CSPOT 7     // potenciometry

#define HzON 1   // 400 Hz ON
#define HzOFF 2  // 400 Hz OFF
#define USB   3   //USB
#define RS485 4   //485
#define REON  5   // Rele ON
#define REOFF 6   // Rele OFF
#define DACL  7   // DAC Low
#define DACH  8   // DAC High
#define USS   10  // 
#define DBZR  107  // zkladn dB 



static const char Cptr[] = {0x0, 0x1, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9};
static const char Sptr0[] = {0x80, 0xC5, 0xCC, 0xCF};
signed char cptr = 1, dptr = 0;
char str[24] = "";
char txstr[24] = "";

char temp[24];
char but = 0;
char leds = 0xFF;
char fptr = 0;
char release = 1;
char dly = 0;
char disp = 0, enc = 0;


unsigned long oscil = 0x225C0128; //zkladn hodnota osc.
unsigned long freq = 1000000; //0x000F4240; //1000000 MHz

unsigned long prod = 1;
char menu = 0, mod = 0;
char sel[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
char slev = 100, slevdb = 20, sptr = 0;
char plev = 15, ppwm = 10;
int ilev = 0, ilevdb = 0, indb = 0;
char amod = 100, mlev = 30;
char ledlev = 30;
char tflg = 0;
char updn = 0, adv = 1, mxip = 0, dac = 0;
char mxi[] = {'i', 'e', 'u', 'f', 'p', ' ', ' ', ' '};
char baud = 5, usb485 = 1, ulev = 0, usbflg = 0, releflag = 0;
char buffer[BFS];
char txptr = 0;
char wptr = 0;
char rptr = 0;
char dsize = 0;
char pchr = 0;
char ech = 0;
char ampl = 0;
char ddsw[5] = {0, 0xF0, 0x08, 0x00, 0};

char multCount;
char rxstr[24] = "";
char rxptr = 0;


__EEPROM_DATA(20, 15, 9, 100, 0, 0, 0, 1);
__EEPROM_DATA(0, 4, 0, 20, 0, 0, 0, 0);
__EEPROM_DATA(0x40, 0x42, 0x0F, 0, 0, 0, 0, 0);
__EEPROM_DATA(0x28, 0x01, 0x5C, 0x22, 0, 0, 0, 0); //0x70,0x91,0xCA,0x23 

void DDSMULT();

void EWrite(char adr, char val) {
    char gint = GIE;
    EEADR = adr;
    EEDATA = val;
    EEPGD = 0;
    CFGS = 0;
    WREN = 1;
    GIE = 0;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    WR = 1;
    while (WR);
    GIE = gint;
    WREN = 0;
}

char ERead(char adr) {
    char data;
    EEADR = adr;
    EEPGD = 0;
    CFGS = 0;
    RD = 1;
    data = EEDATA;
    return data;
}

void Readeprom(void) {
    if (ERead(EPB + 9) > 8) return; //chyba v EPROM

    slevdb = ERead(EPB);
    plev = ERead(EPB + 1);
    ppwm = ERead(EPB + 2);
    amod = ERead(EPB + 3);
    mlev = ERead(EPB + 4);
    mxip = ERead(EPB + 5);
    mxip &= 7;
    dac = ERead(EPB + 6);
    usb485 = ERead(EPB + 7);
    ulev = ERead(EPB + 8);
    baud = ERead(EPB + 9);
    ampl = ERead(EPB + 10);
    ledlev = ERead(EPB + 11);
    freq = 0;
    oscil = 0;
    for (signed char x = 3; x >= 0; x--) {
        freq <<= 8;
        freq += (unsigned long) ERead(EPB + 16 + x);
    }
    for (signed char x = 3; x >= 0; x--) {
        oscil <<= 8;
        oscil += (unsigned long) ERead(EPB + 24 + x);
    }


}

void Writeprom() {

    EWrite(EPB, slevdb);
    EWrite(EPB + 1, plev);
    EWrite(EPB + 2, ppwm);
    EWrite(EPB + 3, amod);
    EWrite(EPB + 4, mlev);
    EWrite(EPB + 5, mxip);
    EWrite(EPB + 6, dac);
    EWrite(EPB + 7, usb485);
    EWrite(EPB + 8, ulev);
    EWrite(EPB + 9, baud);
    EWrite(EPB + 10, ampl);
    EWrite(EPB + 11, ledlev);
    for (char x = 0; x < 4; x++) {
        EWrite(EPB + 16 + x, (char) (freq >> (x * 8)));
    }

}

void Kalibr() {

    for (char x = 0; x < 4; x++) {
        EWrite(EPB + 24 + x, (char) (oscil >> (x * 8)));
    }
}

void Csel(char pin) {
    EN = 1; //diable 173  RC4

    PORTC &= 0xF0;
    PORTC |= pin;
    LDD = 0; //RA5
    LDD = 1;
}

void push_char(char c) {
    wptr++;
    wptr &= (BFS - 1);
    buffer[wptr] = c;
    dsize++;
}

/*
char buffer_full(void) {
    return rptr == wptr && dsize == BFS;
}

char buffer_empty(void) {
    return rptr == wptr && dsize == 0;
}
 */
char pull_char(void) {
    rptr++;
    rptr &= (BFS - 1);
    dsize--;
    return buffer[rptr];
}

void OpenSPI() {
    SSPSTAT &= 0x3F; // power on state
    //SMP = 0; //spi slave

    SSPCON1 = 0x22; //2 = fosc/64 
    CKE = 1; //  1 = transmit 1->0   BF = buffer full   
    CKP = 0; // clock polar 0 = idle
    //BOEN = 1; //ignoruje BF p?ete?en
    // PIE1bits.SSPIE = 1;
    // SSP1CON1bits.SSPEN = 1;
}

void interrupt ISR() {
    static char dsec = 10, chr = 0, tchr = 0;
    if (INT2IF) {
        INT2IF = 0;
        if (PORTAbits.RA2 == 0) {
            if (!PORTAbits.RA3) {
                updn = 1;
            } else {
                updn = 0;
            }
            enc = 1;
        }

    } //if INT2IF

    if (RCIF) {
        chr = RCREG;
        push_char(chr);
    }// RCIF
    if (TXIF) {
        
        tchr = txstr[txptr];
        if (tchr == 0 || tchr == 3 || txptr>15) {
            TXIE = 0; TXEN = 0;
            txptr = 0;
        } else {txptr++; TXREG = tchr; 
        }
       //  TXIE = 0; //TXREG = 0x55;
    }


    if (TMR2IF) {
        TMR2IF = 0;
        if (dsec)dsec--;
        else {
            dsec = 10;
            tflg = 1;
        }
    } //if TMR2IF
} //ISR

void PotSend(char pot, char val) {
    char dm; //pot 0 = slev; pot 1 = mod
    EN = 0;


    SSPBUF = 0x11 + pot;
    while (!SSPIF);
    dm = SSPBUF;
    SSPIF = 0;
    SSPBUF = val;
    while (!SSPIF);
    dm = SSPBUF;
    SSPIF = 0;

    if (pot == 2) {
        SSPBUF = 0;
        while (!SSPIF);
        dm = SSPBUF;
        SSPIF = 0;
        SSPBUF = 0;
        while (!SSPIF);
        dm = SSPBUF;
        SSPIF = 0;
    }

    EN = 1;
}

void DDSINI() {
    char dm;
    EN = 0;
    for (char x = 0; x < 5; x++) {
        SSPBUF = 0;
        while (!SSPIF);
        dm = SSPBUF;
        SSPIF = 0;

    };
    EN = 1;
}

void DDSWR() {
    char dm, wd;
    EN = 0;
    for (char x = 0; x < 5; x++) {
        wd = ddsw[x];
        dm = 0;
        for (char y = 0; y < 8; y++) {
            dm <<= 1;
            dm |= wd & 1;
            wd >>= 1;
        }


        SSPBUF = dm; //ddsw[x];
        while (!SSPIF);
        dm = SSPBUF;
        SSPIF = 0;
    };
    EN = 1;
}

void Puls752(char ppi) {
    PORTCbits.RC7 = 0; //SDO
    SSPCON1 = 0x02;
    //SSPEN = 0;
    Delay(50);
    PORTCbits.RC7 = 0; //SDO
    EN = 0; //   1: 400 Hz on
    for (char x = ppi + 1; x > 0; x--) { //2: 400 Hz off
        PORTCbits.RC7 = 1; //3: USB         
        PORTCbits.RC7 = 0; //4: RS485
    } //5: rele on
    EN = 1; //6: rele off
    SSPCON1 = 0x22; //7: DAC low
    //SSPEN = 1;
} //8: DAC high
//10 - 41   0 - 5V

void BaudSet() {
    char bd = baud;
    SPBRG = Baud[bd << 1];
    SPBRGH = Baud[(bd << 1) + 1];
}

void OpenUART() {
    SPEN = 1;
    SYNC = 0;
    CREN = 1;
    TXIE = 0;
    TXEN = 1;
    BRGH = 0;
    BRG16 = 1;
    BaudSet(); //5 = 19200
    RCIE = 1;
    
}

char ADC(char chan) {

    ADCON0 = 1 + (chan << 2); // AN3, ADON
    GO = 1;
    while (GO);
    return ADRESH;

}

void DDSW() {
    DDSMULT();
    ddsw[4] = 0; //fze
    Csel(FQ);
    DDSWR();
}

void Slevw() {
    char lev;
    Csel(CS752);
    if (slevdb > 20) {
        sptr = slevdb - 20;
        Puls752(REON);
    } else {
        sptr = slevdb;
        Puls752(REOFF);
    };
    lev = dBV1[sptr];

    Delay(100);
    Csel(CSPOT);
     PotSend(0, lev );
     slev = dBV2[sptr];
}

void Ampl() {
    amod = (long) (ampl * 280) / ((ampl + 10));
    Csel(CSPOT);
    PotSend(1, amod);
}

/*
                         Main application
 */
void main(void) {


    PORTC = 0;
    LATB = 0x0;
    LATA = 0x0;
    ANSELH = 0x0;
    LATC = 0x0;
    WPUA = 0x3F;
    WPUB = 0xF0;
    ANSEL = 0x08; //RA4 jako AN3
    ANSELH = 1; // RC6 jako AN8
    TRISB = 0x3F;
    TRISC = 0x40; // RC6 jako AN8
    TRISA = 0x1D;
    GIE = 0;
    EN = 1;
    LDD = 1;

    OSCCON = 0x72;
    OSCTUNE = 0x00;
    /*------- ADC --------*/

    ADCON1 = 0; // poten nastaven
    ADCON2 = 0x26; //fosc/64, TAD=8
    PVCFG1 = 1; //ref = FVR
    FVR1EN = 1;
    FVR1S1 = 1; //4.096V
    FVR1S0 = 1;

    /*------ DAC -------*/

    VREFCON1 = 0xE0;
    VREFCON2 = plev;

    /* -------- PWM */
    CCP1CON = 0x0C; //P1A active high
    PSTRCON = 1; //na RC5         
    PR2 = 249;

    T2CON = 0x4F; //x6;
    /*-----------------*/
    //EWrite(0,12);
    Delay(200);
    Csel(LCD); // cs na LCD
    LCD_Init();
    Readeprom();
    CCPR1L = ledlev;
    if (baud > 9)baud = 8;

    fptr = 1;
    disp = 1;

    prod = 1;
    for (char x = fptr; x < 7; x++)prod *= 10;

    //-------- DDS reset ------
    Csel(RDDS);
    EN = 0;
    asm("NOP");
    asm("NOP");
    EN = 1;
    Delay(100);
    //-------- DDS serial mode ------
    SCLK = 0;
    SDO = 0;
    Csel(FQ);
    EN = 0;
    SCLK = 1;
    asm("NOP");
    asm("NOP");
    SCLK = 0;
    EN = 1;

    OpenSPI();
    Csel(FQ);
    DDSINI();
    DDSMULT();
    ddsw[4] = 0;
    DDSWR();


    OpenUART();
    TMR2IE = 1;

    Delay(255);

    Csel(CS752);

    Puls752(RS485 - usb485);
    Delay(50);
    Puls752(DACL);

    Delay(50);
    Puls752(HzOFF);
    Delay(250);
    Puls752(REOFF);

    //slev = dBV1[slevdb];

    Delay(100);

    Csel(CSPOT);
   // PotSend(0, (slev * 12) / 10);
   //  PotSend(0, slev );
    PotSend(1, Modt[mlev]);
    PotSend(2, Sinp[ppwm]);
    Slevw();
   //  slev = dBV2[slevdb];

    INT2IF = 0;
    INTEDG2 = 0;
    INT2IE = 1;
    PEIE = 1;
    GIE = 1;



    while (1) {

        if (!(rptr == wptr && dsize == 0)) {            
            pchr = pull_char();
            if (pchr == 2)rxptr = 0;
            else {
                rxstr[rxptr] = pchr;
                rxptr++;
                if (rxptr > 16 || pchr == 0 || pchr == 0x0A || pchr == 3) {
                    rxstr[rxptr] = 0;
                    rxptr = 0;
                    switch (rxstr[0] & 0xDF) {
                        case 'R': asm("reset");
                            break;
                        case 'E': Writeprom();
                            break;
                        case 'F': freq = atol(&rxstr[1]);
                            DDSW();
                            break;
                        case 'B': slevdb = (char) (atoi(&rxstr[1]) + 40);
                            Slevw();
                            break;
                        case 'P': plev = (char) atoi(&rxstr[1]);
                            VREFCON2 = plev;
                            break;
                        case 'W': ppwm = (char) atoi(&rxstr[1]);
                            if (ppwm > 10)ppwm = 0;
                            Csel(CSPOT);
                            PotSend(2, Sinp[ppwm]);
                            break;

                        case 'M': mlev = (char) atoi(&rxstr[1]);
                            if (mlev > 20)mlev = 0;
                            Csel(CSPOT);
                            PotSend(1, Modt[mlev]);
                            break;
                        case 'A': ampl = (char) atoi(&rxstr[1]);
                            Ampl();

                            break;
                        case 'U': ulev = (char) atoi(&rxstr[1]);
                            if (ulev > 31)ulev = 0;
                            if (mxip == 2) {
                                Csel(CS752);
                                Puls752(USS + ulev);
                            }
                            break;
                        case 'X': mxip = (char) atoi(&rxstr[1])& 7;
                            if (mxip == 0 && mlev) Puls752(HzON);
                            else Puls752(HzOFF);
                            break;
                        case 'D': dac = (char) atoi(&rxstr[1])& 1;
                            Csel(CS752);
                            if (dac) Puls752(DACH);
                            else Puls752(DACL);
                            break;
                        case 'L': ledlev = (char) atoi(&rxstr[1]);
                            CCPR1L = ledlev;
                            break;
                        case 'Q': txstr[0] = 0x02;
                            itoa(&txstr[1], indb - DBZR, 10);
                            strcat(txstr,"dB\n");                       
                            txptr = 0;                            
                           TXIE = 1;
                           TXEN = 1;
                            break;
                    }//switch
                    disp = 1;
                }
            }
        }


        if (enc) {
            if (sel[mod] == 0) {
                if (updn) {
                    if (freq < 70000000L) freq += prod;
                } else {
                    if (freq >= prod)freq -= prod;
                }
                DDSW();
            }//sel 0
            if (mod == 0) {
                if (sel[0] == 1) {
                    if (updn) {
                        if (slev <= (200 - adv)) slev += adv;
                    } else {
                        if (slev >= adv) slev -= adv;
                    }
                    //doplnit pevod slev na potenc. 
                    /*
                    for (char dp = 0; dp < 27; dp++) {
                        if (dBV1[dp] >= slev) {
                            //   slevdb = dp;
                            break;
                        }
                    }*/
                    Csel(CSPOT);                 
                     PotSend(0, slev );
                }//sel 1
                if (sel[0] == 2) {
                    if (updn) {
                        if (slevdb < 46) slevdb++;
                    } else {
                        if (slevdb > 0) slevdb--;
                    };
                    Slevw();


                }//sel 2
            }//mod 0
            if (mod == 1) {
                if (sel[1] == 1) {
                    if (updn) {
                        if (plev < 31) plev++;
                    } else {
                        if (plev > 0) plev--;
                    }
                    VREFCON2 = plev;
                }
                if (sel[1] == 2) {
                    if (updn) {
                        if (ppwm <= 9) ppwm++;
                    } else {
                        if (ppwm > 0) ppwm--;
                    }
                    Csel(CSPOT);
                    PotSend(2, Sinp[ppwm]);
                }
            }//mod 1
            if (mod == 2) {
                if (sel[2] == 1) {
                    if (updn) {
                        if (ampl <= 100 - adv) ampl += adv;
                    } else {
                        if (ampl >= adv) ampl -= adv;
                    }
                    Ampl();
                }

                if (sel[2] == 2) {
                    if (updn) {
                        if (mlev <= 19) mlev += 1;
                    } else {
                        if (mlev > 0) mlev--;
                    }
                    Csel(CSPOT);
                    PotSend(1, Modt[mlev]);
                }
                if (sel[2] == 3) {
                    Csel(CS752);
                    if (updn) {
                        if (mxip < 5)mxip++;
                    } else {
                        if (mxip > 0)mxip--;
                    }
                    if (mxip == 0 && mlev) Puls752(HzON);
                    else if (mxip > 0)Puls752(HzOFF);
                    //else if(mxip==2);
                }
            }//mod 2

            if (mod == 3) {
                if (sel[3] == 1) {
                    if (updn) {
                        if (usb485 == 0) usb485 = 1;
                    } else {
                        if (usb485 >= 1) usb485 = 0;
                    }
                    usbflg = 1;
                }
                if (sel[3] == 2) {
                    if (updn) {
                        if (baud < 8)baud++;
                    } else {
                        if (baud > 0)baud--;
                    }
                    BaudSet();
                }
            } //mod 3
            if (mod == 4) {
                if (sel[4] == 1) {
                    if (updn) {
                        if (ulev < 31) ulev++;
                    } else {
                        if (ulev > 0)ulev--;
                    }
                    if (mxip == 2) {
                        Csel(CS752);
                        Puls752(USS + ulev);
                    }
                }
                if (sel[4] == 2) {
                    Csel(CS752);
                    if (updn) {
                        dac = 1;
                        Puls752(DACH);
                    } else {
                        dac = 0;
                        Puls752(DACL);
                    }
                }
                if (sel[4] == 3) {
                    Csel(CS752);
                    if (updn) {
                        if (mxip < 5)mxip++;
                    } else {
                        if (mxip > 0)mxip--;
                    }
                    if (mxip == 0 && mlev) Puls752(HzON);
                    else Puls752(HzOFF);
                    //else if(mxip==2);
                }
            }//mod 4 
            if (mod == 6) {
                if (sel[6] == 1) {
                    if (updn) {
                        if (ledlev <= 95) ledlev += 5;
                    } else {
                        if (ledlev >= 5)ledlev -= 5;
                    }
                    CCPR1L = ledlev;
                }
            }

            enc = 0;

            disp = 1;
        }//if enc



        if (tflg) {
            tflg = 0;
            Csel(CST);
            EN = 0;
            //SPI tlatka ---------------           
            SSPBUF = leds;
            while (!SSPIF);
            but = SSPBUF ^0xFF;
            EN = 1;
            SSPIF = 0;
            //---------------------
            //indb = ADC(3);
            ADCON0 = 1 + 0x0C; // AN3, ADON
            GO = 1;
            while (GO);
            indb = ((ADRESL >> 6) + (ADRESH << 2)) / 5;

            if (mod == 5)disp = 1;

            if (dly && release == 0)dly--;
            if (dly == 0)release = 1;
            if (release && but & 0x3F) {
                release = 0;
                if (but & 0x0F) {
                    leds = (but & 0x0F)^0xFF;
                };

                if (mod == 7) {
                    if (but & 8) {
                        Writeprom();
                        Delay(1000);                       
                        leds = 0xB7;
                        goto jmp1;
                    } else if (but & 4) {
                        asm("reset");
                    }
                }
                if (mod == 8 && but & 8) {
                    Kalibr();
                    mod = 0;
                    leds |= 0x80;
                }

                if (but & 1) {
                    if (mod == 0)mod = 8;
                    else mod--;
                    disp = 1;
                }

                if (but & 2) {
                    mod++;
                    if (mod > 8)mod = 0;
                    disp = 1;
                }

                if (but & 4) {
                    if (mod == 2 || mod == 4) {
                        if (sel[mod] == 0) sel[mod] = 3;
                        else if (sel[mod] > 0) sel[mod]--;
                    } else {
                        if (sel[mod] == 0)sel[mod] = 2;
                        else if (sel[mod] > 0) sel[mod]--;
                    }
                    disp = 1;
                }
                if (but & 8) {
                    sel[mod]++;
                    if (mod == 2 || mod == 4) {
                        if (sel[mod] > 3) sel[mod] = 0;
                    } else {
                        if (sel[mod] > 2)sel[mod] = 0;
                    }
                    disp = 1;
                }

                if (sel[mod] == 0) {
                    if ((but & 0x30) == 0x30) {
                        ;
                    } else if (but & 0x10) {
                        fptr++;
                        if (fptr > 7)fptr = 0;
                    } else if (but & 0x20) {

                        if (fptr == 0)fptr = 7;
                        else fptr--;
                    }

                    prod = 1;
                    for (char x = fptr; x < 7; x++)prod *= 10;
                    disp = 1; 
                }//if sel 0

                if (mod == 0 || mod == 2) {
                    if (sel[mod] == 1) {
                        if ((but & 0x30) == 0x30) {
                        } else if (but & 0x10) {
                            cptr++;
                        } else if (but & 0x20) {
                            cptr--;
                        }
                        if (cptr < 0)cptr = 1;
                        if (cptr > 1)cptr = 0;
                        if (cptr)adv = 1;
                        else adv = 10;
                        disp = 1;                        
                    }
                }//mod 0
            }
jmp1:
            if ((but & 0x3F) == 0) {
                release = 1;
                dly = 8;
            }

        }//if tflg
        if (disp) {
            // disp = 0;

            sprintf(str, "%08lu", freq);
            sprintf(temp, "%.2s.%s MHz %c%c", str, &str[2], mlev ? 'm' : ' ', mxi[mxip]);
            Csel(LCD);
            LCD_Chr(0x80, CMD);
            LCD_Strx(temp);

            if (mod == 0) {


                sprintf(str, "%03d", slev);
                if (slevdb > 20) sprintf(temp, "S %c.%s Vp %3d dB ", str[0], &str[1], slevdb - 40);
                else {
                    sprintf(str, "%3d", slev);
                    sprintf(temp, "S %s mVp %3d dB ", str, slevdb - 40);
                };

            } else if (mod == 1) {
                sprintf(str, "%03d", (unsigned) (plev * 500 / 32));
                sprintf(temp, "P  %c.%.2s Vp %02d%%  ", str[0], &str[1], ppwm * 5);
            } else if (mod == 2) {


                sprintf(str, "%03d", ampl);
                sprintf(temp, "M %0.2s.%s Ar %3d%% %c ", &str[0], &str[2], mlev * 5, mxi[mxip]);
            } else if (mod == 3) {
                if (usbflg) {
                    Csel(CS752);
                    Puls752(RS485 - usb485);
                    usbflg = 0;
                }

                sprintf(temp, "C  %s %6s bd  ", BaudText[usb485 + 9], BaudText[baud]);
            } else if (mod == 4) {
                sprintf(str, "%03d", (unsigned) (ulev * 500 / 32));
                sprintf(temp, "U  %c.%.2s V   %c  %c  ", str[0], &str[1], dac ? 'H' : 'L', mxi[mxip]);

            } else if (mod == 5) {
                ilevdb = (int) indb - DBZR + 40;
                if (ilevdb < 0 || ilevdb > 46)ilev = 0;
                else {
                    if (ilevdb > 20)ilev = dBV2[ilevdb - 20] * 10;
                    else ilev = dBV2[ilevdb];
                }
                sprintf(temp, "I %4d mV %3d dB", ilev, ilevdb - 40);
            }//mod 5


            else if (mod == 6)
                sprintf(temp, "L  %3d%%         ", ledlev);

            else if (mod == 7) {
                strcpy(temp, Text[0]);
                leds &= 0x3F;
            } else if (mod == 8) {
                strcpy(temp, Text[1]);
                leds &= 0x7F;
            }

            Csel(LCD);
            LCD_Chr(0xC0, CMD);
            LCD_Strx(temp);

            if (sel[mod] == 0)dptr = Cptr[fptr];

            else if (mod == 0) {
                if (sel[mod] == 1) dptr = cptr - 1;
                else dptr = 0;

            } else if (mod == 2) {
                if (sel[mod] == 1) dptr = Cptr[cptr + 1] - 3;
                else dptr = 0;
            } else dptr = 0;

            LCD_Chr(Sptr0[sel[mod]] + dptr, CMD);

            disp = 0;
        } //if disp


    }//while
}// End Main

void DDSMULT() {
#asm


    clrf _ddsw
    clrf _ddsw + 1
            clrf _ddsw + 2
            clrf _ddsw + 3
            clrf _ddsw + 4
            movlw 0x20
            movwf _multCount
            movf _oscil, w
            movwf _temp
            movf _oscil + 1, w
            movwf _temp + 1
            movf _oscil + 2, w
            movwf _temp + 2
            movf _oscil + 3, w
            movwf _temp + 3
            mult_loop :
            bcf STATUS, 0
            btfss _temp, 0
            goto noAdd
            movf _freq, w
            addwf _ddsw + 1, f
            btfss STATUS, 0
            goto add7
            incfsz _ddsw + 2, f
            goto add7
            incfsz _ddsw + 3, f
            goto add7
            incf _ddsw + 4, f

            add7 : movf _freq + 1, w
            addwf _ddsw + 2, f
            btfss STATUS, 0
            goto add8
            incfsz _ddsw + 3, f
            goto add8
            incf _ddsw + 4, f

            add8 : movf _freq + 2, w
            addwf _ddsw + 3, f
            btfss STATUS, 0
            goto add9
            incf _ddsw + 4, f

            add9 : movf _freq + 3, w
            addwf _ddsw + 4, f

            noAdd : rrcf _ddsw + 4, f
            rrcf _ddsw + 3, f
            rrcf _ddsw + 2, f
            rrcf _ddsw + 1, f
            rrcf _ddsw, f
            rrcf _temp + 3, f
            rrcf _temp + 2, f
            rrcf _temp + 1, f
            rrcf _temp, f
            decfsz _multCount, f
            goto mult_loop
            clrf _ddsw + 4


#endasm
};

